@@ -1,7 +1,7 @@
 /*
  * @(#)Abacus.h
  *
- * Copyright 1994 - 2016  David A. Bagley, bagleyd AT verizon.net
+ * Copyright 1994 - 2019  David A. Bagley, bagleyd AT verizon.net
  *
  * Abacus demo and neat pointers from
  * Copyright 1991 - 1998  Luis Fernandes, elf AT ee.ryerson.ca
@@ -227,31 +227,38 @@ typedef struct {
 #define ACTION_SCRIPT 106
 #define ACTION_MOVE 107
 #define ACTION_PLACE 108
+#define ACTION_HIGHLIGHT_RAIL 109
+#define ACTION_UNHIGHLIGHT_RAIL 110
+#define ACTION_HIGHLIGHT_RAILS 111
 #define ACTION_CLEAR 200
-#define ACTION_COMPLEMENT 201
-#define ACTION_INCREMENT 202
-#define ACTION_DECREMENT 203
-#define ACTION_FORMAT 204
-#define ACTION_ROMAN_NUMERALS 206
-#define ACTION_GROUP 207
-#define ACTION_SIGN 208
-#define ACTION_QUARTER 209
-#define ACTION_TWELFTH 210
-#define ACTION_QUARTER_PERCENT 211
-#define ACTION_SUBDECK 212
-#define ACTION_EIGHTH 213
-#define ACTION_MUSEUM 214
-#define ACTION_ANOMALY 216
-#define ACTION_WATCH 217
-#define ACTION_VERTICAL 218
-#define ACTION_TEACH 219
-#define ACTION_RIGHT_TO_LEFT_ADD 220
-#define ACTION_RIGHT_TO_LEFT_MULT 221
-#define ACTION_SOUND 222
-#define ACTION_SPEED_UP 223
-#define ACTION_SLOW_DOWN 224
-#define ACTION_CLEAR_NODEMO 225
-#define ACTION_TEACH_LINE 226
+#define ACTION_DECIMAL_CLEAR 201
+#define ACTION_COMPLEMENT 202
+#define ACTION_INCREMENT 210
+#define ACTION_DECREMENT 211
+#define ACTION_FORMAT 220
+#define ACTION_ROMAN_NUMERALS 230
+#define ACTION_ANCIENT_ROMAN 231
+#define ACTION_LATIN 232
+#define ACTION_GROUP 233
+#define ACTION_SIGN 240
+#define ACTION_QUARTER 241
+#define ACTION_QUARTER_PERCENT 242
+#define ACTION_TWELFTH 243
+#define ACTION_SUBDECK 250
+#define ACTION_EIGHTH 251
+#define ACTION_MUSEUM 252
+#define ACTION_MODERN_ROMAN 260
+#define ACTION_ANOMALY 261
+#define ACTION_WATCH 262
+#define ACTION_VERTICAL 263
+#define ACTION_TEACH 264
+#define ACTION_RIGHT_TO_LEFT_ADD 270
+#define ACTION_RIGHT_TO_LEFT_MULT 271
+#define ACTION_SOUND 272
+#define ACTION_SPEED_UP 273
+#define ACTION_SLOW_DOWN 274
+#define ACTION_CLEAR_NODEMO 275
+#define ACTION_TEACH_LINE 276
 #define ACTION_DEMO 300
 #define ACTION_NEXT 301
 #define ACTION_REPEAT 302
@@ -286,13 +293,17 @@ typedef struct {
 #define ACTION_ABOUT 903
 #define ACTION_IGNORE 999
 
+#define HIGHLIGHTS_DECK (-12)
+#define UNHIGHLIGHT_DECK (-11)
+#define HIGHLIGHT_DECK (-10)
 #define TEACH_DECK (-9)
 #define CALC_DECK (-8)
 #define MORE_DECK (-7)
 #define JUMP_DECK (-6)
 #define REPEAT_DECK (-5)
 #define NEXT_DECK (-4)
-#define CLEAR_DECK (-3)
+#define CLEAR_DECIMAL_DECK (-3)
+#define CLEAR_DECK (-2)
 #define IGNORE_DECK (-1)
 #define MIN_RAILS 1
 #define MIN_DEMO_RAILS 3
@@ -322,6 +333,7 @@ typedef struct {
 #define COLOR_MIDDLE 1
 #define COLOR_FIRST 2
 #define COLOR_HALF 4
+#define COLOR_GROUP 8
 #define PRIMARY 0
 #define LEFT_AUX 1
 #define RIGHT_AUX 2
@@ -334,9 +346,9 @@ typedef struct {
 #define CHINESE 0
 #define JAPANESE 1
 #define KOREAN 2
-#define ROMAN 3
-#define RUSSIAN 4
-#define DANISH 5
+#define RUSSIAN 3
+#define DANISH 4
+#define ROMAN 5
 #define MEDIEVAL 6
 #define GENERIC 7
 #define MAX_FORMATS 7
