@@ -1,7 +1,7 @@
 /*
  * @(#)AbacusC.c
  *
- * Copyright 1992 - 2016  David A. Bagley, bagleyd AT verizon.net
+ * Copyright 1992 - 2018  David A. Bagley, bagleyd AT verizon.net
  *
  * All rights reserved.
  *
@@ -400,7 +400,7 @@ convertFromInteger(char *outputString, int base, int x)
 	double factor;
 	int fractDigits = currentDigits;
 
-	(void) sprintf(string, "%d", number);
+	intCpy(string, number);
 	if (string[i] == '-') {
 		outputString[l++] = '-';
 		number = -number;
@@ -446,7 +446,13 @@ convertFromDouble(char *outputString, int base, double x)
 	double factor;
 	int fractDigits = currentDigits;
 
-	(void) sprintf(string, "%g", number);
+#ifdef HAVE_SNPRINTF
+	(void) snprintf(string, MAX_VALUE_LENGTH,
+		"%g", number);
+#else
+	(void) sprintf(string,
+		"%g", number);
+#endif
 	if (string[i] == '-') {
 		outputString[l++] = '-';
 		number = -number;
@@ -1009,7 +1015,7 @@ parse(double input, Boolean gotNumeral, char newOperation)
 		pendingOperation = newOperation;
 	}
 	formatToDisplay(memoryBuf, right, currentBase);
-	(void) strcpy(displayBuf, memoryBuf); /* update display */
+	(void) strncpy(displayBuf, memoryBuf, CALC_STRING_SIZE); /* update display */
 }
 
 static void
@@ -1089,7 +1095,7 @@ inputNumeric(char input)
 }
 
 void
-convertStringToAbacus(AbacusWidget w, char *string, int aux)
+convertStringToAbacus(AbacusWidget w, const char *string, int aux)
 {
 	int decimal = 0;
 	int num, factor, i, sign = 0, len = (int) strlen(string);
@@ -1200,7 +1206,7 @@ char *strAddInt(char *buf, int value)
 
 	s = buf;
 	s = s + strlen(buf); /* the position to write */
-	(void) sprintf(s, "%d", value); /* always base 10 */
+	intCpy(s, value); /* always base 10 */
 	return buf;
 }
 
@@ -1224,10 +1230,7 @@ convertStringBases(char *sb, char *buf, int displayBase, int base) {
 			num = convertToDecimal(displayBase,
 				numberString);
 			convertFromDouble(numberString, base, num);
-			(void) strcat(sb, numberString);
-#ifdef DEBUG
-			(void) printf("ns =%s\n", numberString);
-#endif
+			(void) strncat(sb, numberString, CALC_STRING_SIZE - 1);
 		} else {
 			sb = strAddChar(sb, buf[i]);
 		}
@@ -1256,7 +1259,7 @@ void calculate(AbacusWidget w, char * buffer, int aux)
 		(void) printf("conv buffer =%s\n", stringBuf);
 #endif
 	} else {
-		(void) strcpy(stringBuf, buffer);
+		(void) strncpy(stringBuf, buffer, CALC_STRING_SIZE);
 	}
 	resetExpression();
 	for (i = 0; i < strlen(stringBuf); i++) {
