@@ -1,7 +1,7 @@
 /*
  * @(#)AbacusD.c
  *
- * Copyright 1999 - 2018  David A. Bagley, bagleyd AT verizon.net
+ * Copyright 1999 - 2019  David A. Bagley, bagleyd AT verizon.net
  *
  * Abacus demo and neat pointers from
  * Copyright 1991 - 1998  Luis Fernandes, elf AT ee.ryerson.ca
@@ -886,7 +886,7 @@ initializeAbacusDemo(
 	if (abacusLibrary.book != NULL) {
 		fallback = False;
 #ifndef WINVER
-	/* always true in Windows version, some runtime issue with libraries */
+	/* Windows version will crash and burn when it can find XML libraries and compiled in */
 	} else {
 		if (displayedError == 0) {
 			(void) fprintf(stderr, "Trouble using XML, using the static fallback single book demo.\n");
@@ -1102,10 +1102,12 @@ toggleDemoAbacusDemo(AbacusWidget w
 		w->abacusDemo.chapterCount = 0;
 		w->abacusDemo.lessonCount = 0;
 		w->abacusDemo.moveCount = 0;
-		w->abacusDemo.query = True;
+		w->abacusDemo.query = False;
 		w->abacusDemo.chapter = False;
-		w->abacusDemo.book = False;
+		w->abacusDemo.book = True;
+		return;
 	}
+	(void) InvalidateRect(w->core.hWnd, NULL, TRUE);
 #else
 	setAbacusDemo(w, ACTION_DEMO);
 #endif
