@@ -1,7 +1,7 @@
 /*
  * @(#)AbacusM.c
  *
- * Copyright 1994 - 2016  David A. Bagley, bagleyd AT verizon.net
+ * Copyright 1994 - 2018  David A. Bagley, bagleyd AT verizon.net
  *
  * All rights reserved.
  *
@@ -22,7 +22,7 @@
  * AbacusM(ath) handles the string mathematics for the xabacus.
  * Whenever possible the mathematics is done using strings.
  * (Roman math is also done here and is limited by definition.)
- * I use long long when I can not figure out how to do it by strings,
+ * I use long long when I cannot figure out how to do it by strings,
  * as in Display Base.
  */
 #include "AbacusP.h"
@@ -65,8 +65,13 @@ Z = 1/12 ounce
 */
 
 #define MAX_TWELFTH 9
+#define MAX_TWELFTH_GLYPHS 5
+#define MAX_HALFTWELFTH 10 
+#define MAX_HALFTWELFTH_GLYPHS 3
 #define MAX_SUBTWELFTH 18
+#define MAX_SUBTWELFTH_GLYPHS 3 
 #define MAX_SUBEIGHTH 18
+#define MAX_SUBEIGHTH_GLYPHS 3 
 #define MAX_ROMANFRACT 36 /* MAX_TWELFTH+MAX_SUBTWELFTH+strlen(HALF_UNCIA) */
 #define MAX_ROMANLEN 28 /* strlen(LONGEST_ROMANNUMERAL) */
 #define MAX_ROMAN (MAX_ROMANFRACT+MAX_ROMANLEN)
@@ -77,7 +82,7 @@ static char twelfthStrings[][MAX_TWELFTH] = {
 	"triens", "quincunx", "semis", "septunx",
 	"bes", "dodrans", "dextans", "deunix",
 	"as"};
-static char twelfthGlyphs[][5] = {
+static char twelfthGlyphs[][MAX_TWELFTH_GLYPHS] = {
 	"", "-", "=", "=-",
 	"==", "=-=", "S", "S-",
 	"S=", "S=-", "S==", "S=-=",
@@ -85,9 +90,9 @@ static char twelfthGlyphs[][5] = {
 
 #define HALF_UNCIA "semuncia" /* E, (actually a Greek letter sigma) */
 #define ONEANDAHALF_UNCIA "sescuncia" /* E- */
-static char halftwelfthStrings[][10] = {
+static char halftwelfthStrings[][MAX_HALFTWELFTH] = {
 	HALF_UNCIA, ONEANDAHALF_UNCIA};
-static char halftwelfthGlyphs[][3] = {
+static char halftwelfthGlyphs[][MAX_HALFTWELFTH_GLYPHS] = {
 	"E", "E-"};
 
 /* Fractions of Uncia had these names, took shortest variant */
@@ -111,10 +116,10 @@ static char subtwelfthStrings[][MAX_SUBTWELFTH] = {
 	QUARTER_UNCIA, THIRD_UNCIA, FIVETWELFTHS_UNCIA};
 static char subeighthStrings[][MAX_SUBEIGHTH] = {
 	"", EIGHTH_UNCIA, QUARTER_UNCIA, THREEEIGHTHS_UNCIA};
-static char subtwelfthGlyphs[][3] = {
+static char subtwelfthGlyphs[][MAX_SUBTWELFTH_GLYPHS] = {
 	"", "z", "Z",
 	"Q", "u", "QZ"};
-static char subeighthGlyphs[][3] = {
+static char subeighthGlyphs[][MAX_SUBEIGHTH_GLYPHS] = {
 	"", "t", "Q", "Qt"};
 
 int
@@ -307,7 +312,13 @@ dividePieces(char *buf, int base, int pieces, int mult, int places,
 #endif
 	digit = mult / pieces;
 	inter = mult % pieces;
-	(void) sprintf(buf, "%1d%c", digit, decimalPoint);
+#ifdef HAVE_SNPRINTF
+	(void) snprintf(buf, 3,
+		"%1d%c", digit, decimalPoint);
+#else
+	(void) sprintf(buf,
+		"%1d%c", digit, decimalPoint);
+#endif
 	for (position = 2; position < places;) {
 		digit = (inter * base) / pieces;
 		inter = (inter * base) % pieces;
@@ -423,9 +434,9 @@ addStrings(char *buf, char *aBuf, char *bBuf, int base, char decimalPoint)
 		}
 		if (carry > 0) {
 			buf[aSign] = (char) (carry + '0');
-			(void) strcpy(&buf[aSign + 1], &aBuf[aSign]);
+			(void) strncpy(&buf[aSign + 1], &aBuf[aSign], (unsigned int) (aEnd - aSign + 1));
 		} else
-			(void) strcpy(buf, aBuf);
+			(void) strncpy(buf, aBuf, (unsigned int) (aEnd + 1));
 	} else {
 		for (i = bEnd - 1; i > bDecimal; i--) {
 			digit = char2Int(bBuf[i]) +
@@ -446,9 +457,9 @@ addStrings(char *buf, char *aBuf, char *bBuf, int base, char decimalPoint)
 		}
 		if (carry > 0) {
 			buf[bSign] = (char) (carry + '0');
-			(void) strcpy(&buf[bSign + 1], &bBuf[bSign]);
+			(void) strncpy(&buf[bSign + 1], &bBuf[bSign], (unsigned int) (bEnd - bSign + 1));
 		} else
-			(void) strcpy(buf, bBuf);
+			(void) strncpy(buf, bBuf, (unsigned int) (bEnd + 1));
 	}
 	i = (int) strlen(buf) - 1;
 	while (buf[i] == '0' && buf[i - 1] != decimalPoint)
@@ -561,8 +572,8 @@ convertString(char * buf, char * inbuf, int base, int displayBase,
 	(void) int2String(buf, intPart, displayBase, negative);
 	(void) flt2String(fltbuf, floatPart, base, displayBase,
 		decimalPointPlace, decimalPosition);
-	(void) strcat(buf, ".");
-	(void) strcat(buf, fltbuf);
+	(void) strncat(buf, ".", 2);
+	(void) strncat(buf, fltbuf, STRING_SIZE);
 #ifdef DEBUG
 	(void) printf("convertString: outbuf %s\n", buf);
 #endif
@@ -589,49 +600,52 @@ romanFraction(char *buf, int base, int number, int subnumber,
 	int halfBase = subbase / 2;
 	int fraction = number, subfraction = subnumber;
 
-	fraction %= base;
+	if (base < ROMANFRACTIONBASE) {
+		fraction *= ROMANFRACTIONBASE / base;
+		subfraction *= ROMANFRACTIONBASE / base;
+		fraction += (subfraction / subbase);
+		subfraction = subfraction % subbase;
+	}
 	if (fraction == 1 && subfraction >= halfBase) {
 		subfraction -= halfBase;
 		if (latin) {
-			(void) strcat(buf, halftwelfthStrings[1]);
+			(void) strncat(buf, halftwelfthStrings[1], MAX_HALFTWELFTH);
 			gotFraction = True;
 		} else {
-			(void) strcat(buf, halftwelfthGlyphs[1]);
+			(void) strncat(buf, halftwelfthGlyphs[1], MAX_HALFTWELFTH_GLYPHS);
 		}
 	} else if (fraction > 0) {
 		if (latin) {
-			(void) strcat(buf, twelfthStrings[fraction *
-				ROMANFRACTIONBASE / base]);
+			(void) strncat(buf, twelfthStrings[fraction], MAX_TWELFTH);
 			gotFraction = True;
 		} else {
-			(void) strcat(buf, twelfthGlyphs[fraction *
-				ROMANFRACTIONBASE / base]);
+			(void) strncat(buf, twelfthGlyphs[fraction], MAX_TWELFTH_GLYPHS);
 		}
 	}
 	if (subfraction >= halfBase) {
 		subfraction -= halfBase;
 		if (latin) {
 			if (gotFraction)
-				(void) strcat(buf, " ");
+				(void) strncat(buf, " ", 2);
 			gotFraction = True;
-			(void) strcat(buf, halftwelfthStrings[0]);
+			(void) strncat(buf, halftwelfthStrings[0], MAX_HALFTWELFTH);
 		} else {
-			(void) strcat(buf, halftwelfthGlyphs[0]);
+			(void) strncat(buf, halftwelfthGlyphs[0], MAX_HALFTWELFTH_GLYPHS);
 		}
 	}
 	if (subfraction != 0) {
 		if (latin) {
 			if (gotFraction)
-				(void) strcat(buf, " ");
+				(void) strncat(buf, " ", 2);
 			if (subbase == 8)
-				(void) strcat(buf, subeighthStrings[subfraction]);
+				(void) strncat(buf, subeighthStrings[subfraction], MAX_SUBEIGHTH);
 			else
-				(void) strcat(buf, subtwelfthStrings[subfraction]);
+				(void) strncat(buf, subtwelfthStrings[subfraction], MAX_SUBTWELFTH);
 		} else {
 			if (subbase == 8)
-				(void) strcat(buf, subeighthGlyphs[subfraction]);
+				(void) strncat(buf, subeighthGlyphs[subfraction], MAX_SUBEIGHTH_GLYPHS);
 			else
-				(void) strcat(buf, subtwelfthGlyphs[subfraction]);
+				(void) strncat(buf, subtwelfthGlyphs[subfraction], MAX_SUBTWELFTH_GLYPHS);
 		}
 	}
 }
@@ -641,7 +655,7 @@ append(char *buf, int position, int place, Boolean ancient)
 {
 	if (ancient) {
 		buf[position] = '\0';
-		(void) strcat(&buf[position], oldRoman[place]);
+		(void) strncat(&buf[position], oldRoman[place], MAX_OLDROMAN);
 		return (int) strlen(oldRoman[place]);
 	} else {
 		buf[position] = roman[place];
@@ -650,7 +664,7 @@ append(char *buf, int position, int place, Boolean ancient)
 }
 
 int
-string2Roman(char *buf, char *inbuf, int base,
+string2Roman(char *buf, char *inbuf, int base, Boolean decimalFraction,
 		int pieces, int number, int subnumber, int subbase,
 		char decimalPoint, Boolean ancientRoman, Boolean latin)
 {
@@ -660,6 +674,8 @@ string2Roman(char *buf, char *inbuf, int base,
 	buf[position] = '[';
 	position++;
 	if (inbuf[i] == '-') {
+		buf[position] = '?';
+		position++;
 		buf[position] = ']';
 		position++;
 		buf[position] = '\0';
@@ -668,14 +684,15 @@ string2Roman(char *buf, char *inbuf, int base,
 	last = (int) strlen(inbuf);
 	for (i = 0; i < last; i++) {
 		if (inbuf[i] == decimalPoint) {
+			last = i;
 			break;
 		}
 	}
-	last = i;
 	i = 0;
 	digit = char2Int(inbuf[i]);
 	if (last > loga || (last == loga &&
-			digit >= ((base >= 8) ? base / 2 - 1 : base))) {
+			digit >= ((base >= 8) ? base / 2 - 1 : base))
+			|| decimalFraction) {
 		buf[position] = '?';
 		position++;
 		buf[position] = ']';
@@ -729,15 +746,13 @@ string2Roman(char *buf, char *inbuf, int base,
 		char fractbuf[MAX_ROMANFRACT] = "";
 		int subfraction = subnumber;
 
-		if (pieces != ROMANFRACTIONBASE)
-			subfraction = 0; /* words not scalable */
 		if (latin && position > 1 && (number != 0 || subfraction != 0))
-			(void) strcat(buf, " ");
+			(void) strncat(buf, " ", 2);
 		romanFraction(fractbuf, pieces, number, subfraction,
 			subbase, latin);
-		(void) strcat(buf, fractbuf);
+		(void) strncat(buf, fractbuf, MAX_ROMANFRACT);
 	}
-	(void) strcat(buf, "]");
+	(void) strncat(buf, "]", 2);
 	return 0;
 }
 
@@ -800,8 +815,9 @@ int2Roman(char *buf, int arabic)
 void string2Group(char *buf, char *inbuf, int groupSize, char decimalPoint,
 		char groupSeparator) {
 	int i, j, len;
+	int max = (int) strlen(inbuf);
 
-	for (i = 0; i < (int) strlen(inbuf); i++) {
+	for (i = 0; i < max; i++) {
 		if (inbuf[i] == ' ' || inbuf[i] == decimalPoint) {
 			break;
 		}
@@ -821,7 +837,7 @@ void string2Group(char *buf, char *inbuf, int groupSize, char decimalPoint,
 		j++;
 	}
 	buf[j] = '\0';
-	(void) strcat(buf, &inbuf[len - 1]);
+	(void) strncat(buf, &inbuf[len - 1], (unsigned int) max);
 }
 
 int
