@@ -1,7 +1,7 @@
 /*
  * @(#)AbacusT.c
  *
- * Copyright 2009 - 2018  David A. Bagley, bagleyd AT verizon.net
+ * Copyright 2009 - 2019  David A. Bagley, bagleyd AT verizon.net
  *
  * All rights reserved.
  *
@@ -1380,7 +1380,7 @@ static Boolean pendingUpdate(AbacusWidget w, char * buffer,
 			strAddInt(buffer, ABS(w->abacus.lower));
 			if (bottomNumber <= base / 2) {
 				if (w->abacus.lower < 0)
-					(void) strncat(buffer, " from lower deck", 17); 
+					(void) strncat(buffer, " from lower deck", 17);
 				else
 					(void) strncat(buffer, " to lower deck", 15);
 			}
@@ -2259,9 +2259,21 @@ void teachStep(AbacusWidget w, char * buffer, int aux)
 	double a, b;
 
 #ifdef HAVE_GETTEXT
-	setlocale(LC_ALL, "");
-	/*bindtextdomain("AbacusT", getenv("PWD"));*/
-	bindtextdomain("AbacusT", "/usr/share/locale/");
+#ifndef GETTEXTPATH
+#ifdef VMS
+#define GETTEXTPATH "[.]"
+#else
+#if 0
+#define GETTEXTPATH "/usr/share/locale"
+#else
+#define GETTEXTPATH "."
+#endif
+#endif
+#endif
+	if (setlocale(LC_ALL, "") == NULL) {
+		(void) printf("Unable to set locale");
+	}
+	bindtextdomain("AbacusT", GETTEXTPATH);
 	textdomain("AbacusT");
 #endif
 
@@ -2289,8 +2301,8 @@ void teachStep(AbacusWidget w, char * buffer, int aux)
 #ifdef OLD_DEBUG
 		testTable();
 #endif
-		drawLineText(w, (char *) TEACH_STRING0, 0);
-		drawLineText(w, (char *) TEACH_STRING1, 1);
+		drawLineText(w, TEACH_STRING0, 0);
+		drawLineText(w, TEACH_STRING1, 1);
 		(void) sprintf(buffer3, "(addition order will be: %s; multiplicand order will be %s)",
 			((w->abacus.rightToLeftAdd) ? "right to left" : "left to right"),
 			((w->abacus.rightToLeftMult) ? "right to left" : "left to right"));
@@ -2379,7 +2391,8 @@ void teachStep(AbacusWidget w, char * buffer, int aux)
 			decimalSafe(w->abacus.bString);
 			addSafe(w->abacus.aString, w->abacus.bString);
 			(void) strcpy(w->abacus.rString, w->abacus.aString);
-			(void) sprintf(buffer3, "Current answer: %s  ",
+			(void) sprintf(buffer3, "%s %s",
+				_("Current answer:"),
 				w->abacus.rString);
 			drawLineText(w, buffer3, 2);
 			w->abacus.step++;
@@ -2414,7 +2427,8 @@ void teachStep(AbacusWidget w, char * buffer, int aux)
 			decimalSafe(w->abacus.aString);
 			decimalSafe(w->abacus.bString);
 			(void) strncpy(w->abacus.rString, "0.", 3);
-			(void) sprintf(buffer3, "Current answer: %s  ",
+			(void) sprintf(buffer3, "%s %s",
+				_("Current answer:"),
 				w->abacus.rString);
 			drawLineText(w, buffer3, 2);
 			w->abacus.step++;
@@ -2457,7 +2471,8 @@ void teachStep(AbacusWidget w, char * buffer, int aux)
 			decimalSafe(w->abacus.bString);
 			(void) strcpy(w->abacus.rString, w->abacus.aString);
 			(void) strncpy(w->abacus.sString, "0.", 3);
-			(void) sprintf(buffer3, "Current answer: %s  ",
+			(void) sprintf(buffer3, "%s %s",
+				_("Current answer:"),
 				w->abacus.sString);
 			drawLineText(w, buffer3, 2);
 			w->abacus.step++;
@@ -2489,7 +2504,8 @@ void teachStep(AbacusWidget w, char * buffer, int aux)
 			drawLineText(w, buffer1, 0);
 			(void) strcpy(w->abacus.rString, w->abacus.aString);
 			(void) strncpy(w->abacus.sString, "0.", 3);
-			(void) sprintf(buffer3, "Current answer: %s  ",
+			(void) sprintf(buffer3, "%s %s",
+				_("Current answer:"),
 				w->abacus.sString);
 			drawLineText(w, buffer3, 2);
 			w->abacus.step++;
@@ -2766,7 +2782,8 @@ void teachStep(AbacusWidget w, char * buffer, int aux)
 					w->abacus.lower, w->abacus.upper);
 				w->abacus.reg = -1;
 				contractStringBuffer(w->abacus.sString);
-				(void) sprintf(buffer3, "Current answer: %s  ",
+				(void) sprintf(buffer3, "%s %s",
+					_("Current answer:"),
 					w->abacus.sString);
 				drawLineText(w, buffer3, 2);
 				return;
@@ -2797,7 +2814,8 @@ void teachStep(AbacusWidget w, char * buffer, int aux)
 					w->abacus.lower, w->abacus.upper);
 				w->abacus.reg = -1;
 				contractStringBuffer(w->abacus.sString);
-				(void) sprintf(buffer3, "Current answer: %s  ",
+				(void) sprintf(buffer3, "%s %s",
+					_("Current answer:"),
 					w->abacus.sString);
 				drawLineText(w, buffer3, 2);
 				return;
@@ -2858,10 +2876,12 @@ void teachStep(AbacusWidget w, char * buffer, int aux)
 			if (w->abacus.op == '/' ||
 					w->abacus.op == 'v' || w->abacus.op == 'u') {
 				contractStringBuffer(w->abacus.sString);
-				(void) sprintf(buffer3, "Current answer: %s  ",
+				(void) sprintf(buffer3, "%s %s",
+					_("Current answer:"),
 					w->abacus.sString);
 			} else {
-				(void) sprintf(buffer3, "Current answer: %s  ",
+				(void) sprintf(buffer3, "%s %s",
+					_("Current answer:"),
 					w->abacus.rString);
 			}
 			drawLineText(w, buffer3, 2);
